restart; with(plots):
Su := [Cx=0.64, r=0.1925, o=2.98, Xv=2., Yv=-0.2, alpha=1*pi/180,
        beta=35*pi/180, S=1.3, s[1]=0.1, s[2]=0.3, s[3]=1.085,
        h=0.4, pi=evalf(Pi)];
assign(Su);
omega:=2*pi*o; v:=omega*r:
L := [Cx, r*pi, Cx, r*pi];
L := [seq(sum(L[i], i=1..j), j=1..4)];
Tau := map(u->u/v,L);
Tf := Tau[4]:
Gx[1] := v*t: Gy[1]:=r:                    #motion to right
Gx[2] := Cx+r*cos(pi/2-omega*(t-Tau[1])):
Gy[2] := r*sin(pi/2-omega*(t-Tau[1])):     #right semicircle
Gx[3] := Cx-v*(t-Tau[2]): Gy[3]:=-r:       #motion to left
Gx[4] := r*cos(3/2*pi-omega*(t-Tau[3])):
Gy[4] := r*sin(3/2*pi-omega*(t-Tau[3])):   #left semicircle
Gx := piecewise(seq([t<=Tau[j], Gx[j]][],j=1..4),
       subs(t=t-Tf,Gx[1]));
Gy := piecewise(seq([t<=Tau[j], Gy[j]][], j=1..4),
       subs(t=t-Tf,Gy[1]));
AP[1] := plot([Gx, Gy, t=0..Tf], scaling=constrained,
         labels=["x [m]", "y [m]"]): %;
Line := y=k*x+q; # line - general equation
L1 := subs(K=tan(alpha),
      [k=K, q=solve(subs(x=Xv, y=Yv, k=K, Line), q)]);
L2 := subs(K=tan(beta),
      [k=K, q=solve(subs(x=Xv, y=Yv, k=K,Line), q)]);
e := (Gx-Xv)^2+(Gy-Yv)^2-S^2:
AP[2] := plot(e, t=0..Tf, labels=["t [s]","[m^2]"]): %;
tau := [fsolve(e, t=0.2..0.3), fsolve(e, t=0.3..0.4)];
eK := (X-x)^2+(Y-y)^2 = S^2:
sol := [allvalues(solve({eK, Line}, {x, y}))]:
solf := evalf(subs(X=Gx, Y=Gy, L2, t=(Tau[1]+Tau[2])*0.5, sol));
solf := map(u -> evalb(subs(u, x-Xv) > 0), solf);
sol := normal(zip((u, v) -> `if`(u, v, NULL), solf, sol)[]);
Hx := subs(X=X(t), Y=Y(t), piecewise(t<tau[1], subs(sol, L1, x),
      t<tau[2], subs(sol, L2, x), subs(sol, L1, x))):
Hy := subs(X=X(t), Y=Y(t), piecewise(t<tau[1], subs(sol, L1, y),
      t<tau[2], subs(sol, L2, y), subs(sol, L1, y))):
AP[3] := plot(subs(X(t)=Gx, Y(t)=Gy, [Hx, Hy, t=0..Tf]),
      color=black, labels=["x [m]","y [m]"]): %;
DSu := [X(t)='Gx', diff(X(t), t)=Gxt, diff(X(t), t, t)=Gxtt,
        Y(t)='Gy', diff(Y(t), t)=Gyt, diff(Y(t), t, t)=Gytt];
dsu := [x(t)='Hx', diff(x(t), t)=Hxt, diff(x(t), t, t)=Hxtt,
        y(t)='Hy', diff(y(t), t)=Hyt, diff(y(t), t, t)=Hytt];
Gxt := diff(Gx, t):       Hxt := subs(DSu, diff(Hx, t)):
Gxtt := diff(Gx, t, t):   Hxtt := subs(DSu, diff(Hx, t, t)):
Gyt := diff(Gy,t):        Hyt := subs(DSu, diff(Hy, t)):
Gytt := diff(Gy, t, t):   Hytt := subs(DSu, diff(Hy, t, t)):
Opt:=color=black,thickness=3,linestyle=1:
PL1 := plot([Hxt, Hyt], t=0..Tf, Opt):
PL2 := plot([Hxtt, Hytt], t=0..Tf, Opt):
save PL1, PL2, "plots.sav":
Ex := (x(t)-X(t))/S: Ey := (y(t)-Y(t))/S;
for j from 1 to 3 do;
   Px[j] := X(t)+Ex*s[j]+Ey*h;    Py[j] := Y(t)+Ey*s[j]-Ex*h;
   Vx[j] := diff(Px[j],t);        Vy[j] := diff(Py[j],t);
   Ax[j] := diff(Vx[j],t);        Ay[j] := diff(Vy[j],t);
   V[j] := sqrt(Vx[j]^2+Vy[j]^2); A[j] := sqrt(Ax[j]^2+Ay[j]^2);
   At[j] := diff(V[j],t);         An[j] := sqrt(A[j]^2-At[j]^2);
end do:
PlotOpt := color=[black, black, black, grey, grey, grey],
         thickness=[3, 2, 1, 3, 2, 1]:
AP[4] := plot(subs(dsu, DSu, [seq([Px[j], Vx[j], t=0..Tf],
         j=1..3), seq([Py[j], Vy[j], t=0..Tf], j=1..3)]),
         PlotOpt, labels=["x,y [m]", "Vx,Vy [m/s]"]): %;
AP[5] := plot(subs(dsu, DSu, [seq([Px[j], Ax[j], t=0..Tf],
         j=1..3), seq([Py[j], Ay[j], t=0..Tf], j=1..3)]),
         PlotOpt, labels=["x,y [m]", "Ax,Ay [m/s^2]"]): %;
AP[6] := plot(subs(dsu, DSu, [seq([Vx[j], Ax[j], t=0..Tf],
         j=1..3), seq([Vy[j], Ay[j], t=0..Tf], j=1..3)]),
         PlotOpt, labels=["Vx,Vy [m/s]", "Ax,Ay [m/s^2]"]): %;
AP[7] := plot(subs(dsu, DSu, [seq(Px[j], j=1..3), seq(Py[j],
         j=1..3)]), t=0..Tf, PlotOpt,
         labels=["t [s]", "x,y [m]"]): %;
AP[8] := plot(subs(dsu, DSu, [seq(Vx[j], j=1..3),
         seq(Vy[j], j=1..3)]), t=0..Tf, PlotOpt,
         labels=["t [s]", "Vx,Vy [m/s]"]): %;
AP[9] := plot(subs(dsu, DSu, [seq(Ax[j], j=1..3),
         seq(Ay[j], j=1..3)]), t=0..Tf, PlotOpt,
         labels=["t [s]", "Ax,Ay [m/s^2]"]): %;
AP[10] := plot(subs(dsu, DSu, [seq(At[j], j=1..3),
         seq(An[j], j=1..3)]), t=0..Tf, PlotOpt,
         labels=["t [s]", "At,An [m/s^2]"]): %;
AP[11] := plot(subs(dsu, DSu, [seq([V[j], A[j], t=0..Tf],
         j=1..3)]), PlotOpt, labels=["|V| [m/s]",
         "|A| [m/s^2]"]): %;
A1 := display([seq(plot(subs(dsu, DSu, t=tt, {[[Gx,Gy], [Hx,Hy]],
      seq([[Px[j], Py[j]], [Px[j]-Ey*h, Py[j]+Ex*h]], j=1..3)}),
      color=brown), tt=[seq(Tf*i/200, i=0..200)])],
      insequence=true,thickness=3,scaling=constrained):
A2 := display([seq(plot(subs(dsu, DSu, [seq([Px[j], Py[j],
      t=0..Tf*i/200], j=1..3)]), color=[black, blue, red]),
      i=1..200)], insequence=true, thickness=3,
      scaling=constrained):
A3 := plot(subs(dsu, DSu, {[Gx, Gy, t=0..Tf], [Hx, Hy,
      t=0..Tf]}), scaling=constrained, color=grey, thickness=3):
AT := display({A1, A2, A3}):
AT;
plotsetup(gif, plotoutput=`animation.gif`,
  plotoptions=`portrait, noborder, height=600,
               width=800, colors=16`);
AT;
save Gx, Gy, Gxt, Gyt, Gxtt, Gytt, Tf, v, "drive.sav":
for j from 1 to 11 do;
    plotsetup(ps, plotoutput=cat(ap, j, `.ps`),
       plotoptions=`landscape, noborder`);
    AP[j];
end do;
